@@ -9,9 +9,8 @@ import json
 import math
 import os
 import time
-from asyncio import exceptions
 from threading import RLock
-from typing import Any, ClassVar
+from typing import Any, ClassVar,Optional
 
 import numpy as np
 import pandas as pd
@@ -23,6 +22,7 @@ from pydantic import BaseModel, Field, field_validator
 
 from mainsequence.logconf import logger
 
+from . import exceptions
 from .base import TDAG_ENDPOINT, BaseObjectOrm, BasePydanticModel
 from .data_sources_interfaces import timescale as TimeScaleInterface
 from .data_sources_interfaces.duckdb import DuckDBInterface
@@ -89,8 +89,8 @@ class SourceTableConfigurationDoesNotExist(Exception):
 
 
 class ColumnMetaData(BasePydanticModel, BaseObjectOrm):
-    source_config_id: int = Field(
-        ...,
+    source_config_id: Optional[int] = Field(
+        None,
         alias="source_config",
         description="Primary key of the related SourceTableConfiguration",
     )
@@ -1458,6 +1458,28 @@ class UpdateStatistics(BaseModel):
 
         return self._max_time_in_update_statistics
 
+    @property
+    def is_any_asset_on_fallback_date(self)->bool:
+        """"
+        return true if any of the assets in asset_time_statistics equals _initial_fallback_date
+        """
+
+
+        for k,v in self.asset_time_statistics.items():
+            if v==self._initial_fallback_date:
+                return True
+        return False
+    @property
+    def are_all_assets_on_fallback_date(self)->bool:
+        """"
+             return true if all assets in asset_time_statistics equals _initial_fallback_date
+             """
+        for k,v in self.asset_time_statistics.items():
+            if v!=self._initial_fallback_date:
+                return False
+        return True
+
+
     def get_update_range_map_great_or_equal_columnar(
         self,
         extra_time_delta: datetime.timedelta | None = None,
@@ -1762,7 +1784,7 @@ class UpdateStatistics(BaseModel):
     def filter_df_by_latest_value(self, df: pd.DataFrame) -> pd.DataFrame:
 
 
-
+        a=5
         # Single-index time series fallback
         if "unique_identifier" not in df.index.names:
             if self.max_time_index_value is not None:
@@ -2327,6 +2349,7 @@ class Artifact(BasePydanticModel, BaseObjectOrm):
     created_by_resource_name: str
     bucket_name: str
     content: Any
+    creation_date: datetime.datetime
 
     @classmethod
     def upload_file(cls, filepath, name, created_by_resource_name, bucket_name=None):
